#ifndef AUBO_SCOPE_KEYBOARD_INPUT_VALIDATOR_H
#define AUBO_SCOPE_KEYBOARD_INPUT_VALIDATOR_H

#include <string>
#include <QObject>

#include <aubo_caps/domain/value/expression/expression.h>

namespace arcs {
namespace aubo_scope {

/**
 * <p>
 * Interface representing the input validators created by {@link
 * InputValidationFactory}. This factory provides a set of standard input
 * validators that can be used for validating the input entered by the user
 * using the virtual keyboard/keypad.
 * </p>
 *
 * If the standard validators available in the {@link InputValidationFactory}
 * does not satisfy your needs, you are free to implement your own custom
 * validator.
 *
 * @param <T> the (generic) type parameter for the interface representing the
 * type of input data being validated (e.g. Integer or Double).
 */
template <typename T>
struct InputValidator
{
    /**
     * @param value to be validated.
     * @return {@code true} if value is valid.
     */
    virtual bool isValid(T value) = 0;

    /**
     * Returns a meaningful message in case the value is not valid.
     *
     * @param value the invalid value. Can be included in the message if it
     * makes sense.
     * @return message.
     */
    virtual std::string getMessage(T value) = 0;
};

template <typename T>
struct RangeValidator : InputValidator<T>
{
    RangeValidator(T minValue, T maxValue)
        : min_value_(minValue), max_value_(maxValue)
    {
    }

    bool isValid(T value) override
    {
        return value >= min_value_ && value <= max_value_;
    }

    std::string getMessage(T value) override
    {
        if (value < min_value_ || value > max_value_) {
            if constexpr (std::is_same<T, double>::value) {
                return QObject::tr("Error: The value is between %1 and %2.")
                    .arg(min_value_, 0, 'g', 15)
                    .arg(max_value_, 0, 'g', 15)
                    .toStdString();
            } else {
                return QObject::tr("Error: The value is between %1 and %2.")
                    .arg(min_value_, 0)
                    .arg(max_value_, 0)
                    .toStdString();
            }
        }
        return "";
    }

    T min_value_;
    T max_value_;
};

using IntegerRangeValidator = RangeValidator<int>;
using UIntegerRangeValidator = RangeValidator<uint32_t>;
using DoubleRangeValidator = RangeValidator<double>;

struct StringLengthValidator : InputValidator<std::string>
{
    StringLengthValidator(int minLength, int maxLength)
        : min_length_(minLength), max_length_(maxLength)
    {
    }

    bool isValid(std::string value) override
    {
        return (int)value.length() >= min_length_ &&
               (int)value.length() <= max_length_;
    }

    std::string getMessage(std::string value) override
    {
        if ((int)value.length() < min_length_ ||
            (int)value.length() > max_length_) {
            return QObject::tr(
                       "Error: The value is between %1 and %2 in length.")
                .arg(min_length_)
                .arg(min_length_)
                .toStdString();
        }
        return "";
    }
    int min_length_;
    int max_length_;
};
} // namespace aubo_scope
} // namespace arcs

#endif
