#ifndef AUBO_SCOPE_KEYBOARD_NUMBER_INPUT_H
#define AUBO_SCOPE_KEYBOARD_NUMBER_INPUT_H

#include <string>
#include <memory>
#include <functional>
#include <aubo_caps/domain/user_interface/keyboard_input_callback.h>
#include <aubo_caps/domain/user_interface/input_validator.h>

class QWidget;

namespace arcs {
namespace aubo_scope {

/**
 * Interface representing the virtual numeric keypad used for configuring it and
 * requesting it to be displayed in PolyScope.
 *
 * @tparam <T> The (generic) type parameter for the interface representing the
 * type of input data entered by the end user via the virtual keypad. Relevant
 * types are e.g. {@link Integer} and {@link Double}.
 *
 */
template <typename T>
class ARCS_ABI_EXPORT KeyboardNumberInput
{
public:
    KeyboardNumberInput(KeyboardNumberInput &f);
    KeyboardNumberInput(KeyboardNumberInput &&f);
    virtual ~KeyboardNumberInput();

    /**
     * <p>
     * Sets an input validator for the virtual numeric keypad. This validator
     * will be used to validate the input entered by the end user. Some standard
     * validators are available in {@link
     * userinteraction.inputvalidation.InputValidationFactory}.
     * </p>
     *
     * <p>
     * Specifying an input validator is recommended, but optional.
     * </p>
     *
     * In case {@link InputValidator#isValid(Object)} returns {@code false}, the
     * accept button on the virtual keypad will be disabled and the error
     * message from {@link InputValidator#getMessage(Object)} will be shown.
     *
     * @param validator the validator to be used to validate keypad user input.
     * @return this KeyboardNumberInput object.
     * @throws NullPointerException in case the validator is {@code null}.
     */
    void setErrorValidator(const std::shared_ptr<InputValidator<T>> &validator);

    /**
     * Sets the initial value which will be displayed when the virtual numeric
     * keypad is shown.
     *
     * @param initialValue the initial value for the virtual keypad.
     * @return this KeyboardNumberInput object.
     */
    void setInitialValue(T initialValue);

    /**
     * This method requests the virtual numeric keypad to be shown.
     *
     * @param component the JTextField that will be used to define the screen
     * position of the virtual keypad. The component must be visible on the
     * screen at the time of showing the keypad.
     * @param callback  the callback to be used when the end user exits the
     * keypad by accepting or canceling the input.
     * @throws NullPointerException in case any parameter is {@code null}.
     */
    void show(QWidget *component, const std::function<void(bool, T)> &callback);

private:
    friend class DataSwitch;
    KeyboardNumberInput();
    void *d_{ nullptr };
};

using KeyboardUIntInput = KeyboardNumberInput<uint32_t>;
using KeyboardIntInput = KeyboardNumberInput<int>;
using KeyboardDoubleInput = KeyboardNumberInput<double>;

} // namespace aubo_scope
} // namespace arcs

#endif
