#ifndef AUBO_SCOPE_KEYBOARD_TEXT_INPUT_H
#define AUBO_SCOPE_KEYBOARD_TEXT_INPUT_H

#include <memory>
#include <functional>

#include <aubo_caps/domain/user_interface/keyboard_input_callback.h>
#include <aubo_caps/domain/user_interface/input_validator.h>

class QWidget;

namespace arcs {
namespace aubo_scope {

ARCS_CLASS_FORWARD(KeyboardTextInput);

/**
 * Interface representing the virtual keyboard used for configuring it and
 * requesting it to be displayed in PolyScope.
 */
class ARCS_ABI_EXPORT KeyboardTextInput
{
public:
    KeyboardTextInput(KeyboardTextInput &f);
    KeyboardTextInput(KeyboardTextInput &&f);
    virtual ~KeyboardTextInput();

    /**
     * <p>
     * Sets an input validator for the virtual keyboard. This validator will be
     * used to validate the input entered by the user. Some standard validators
     * are available in
     * {@link domain.userinteraction.inputvalidation.InputValidationFactory}.
     * </p>
     *
     * <p>
     * Specifying an input validator is recommended, but optional.
     * </p>
     *
     * In case {@link InputValidator#isValid(Object)} (in this case parameter is
     * {@link String}) returns {@code false}, the accept button on the virtual
     * keyboard will be disabled and the error message from {@link
     * InputValidator#getMessage(Object)} (parameter is {@link String}) will be
     * shown.
     *
     * @param validator the validator to be used to validate keyboard user
     * input.
     * @return this KeyboardTextInput object.
     * @throws NullPointerException in case the validator is {@code null}.
     */
    void setErrorValidator(
        const std::shared_ptr<InputValidator<std::string>> &validator);

    /**
     * Sets the text which will be displayed when the virtual keyboard is shown.
     *
     * @param initialText the initial text for the virtual keyboard.
     * @return this KeyboardTextInput object.
     */
    virtual void setInitialValue(const std::string &initialText);

    /**
     * This method requests the virtual standard keyboard to be shown.
     *
     * @param component the JTextField that will be used to define the screen
     * position of the virtual keyboard. The component must be visible on the
     * screen at the time of showing the keypad.
     * @param callback  the callback to be used when the user exits the keyboard
     * by accepting or canceling the input.
     * @throws NullPointerException in case any parameter is {@code null}.
     */
    void show(QWidget *component,
              const std::function<void(bool, const std::string &)> &callback);

private:
    friend class DataSwitch;
    KeyboardTextInput();
    void *d_{ nullptr };
};
} // namespace aubo_scope
} // namespace arcs

#endif
