#ifndef AUBO_SCOPE_USER_INTERACTION_H
#define AUBO_SCOPE_USER_INTERACTION_H

#include <vector>
#include <functional>
#include <aubo_caps/class_forward.h>
#include <aubo_caps/domain/user_interface/keyboard_input_factory.h>

class QPixmap;
namespace arcs {
namespace aubo_scope {

ARCS_CLASS_FORWARD(UserInteraction);
ARCS_CLASS_FORWARD(TCP);

using RobotPositionCallback = std::function<void(
    bool accepted, const std::vector<double> &pose,
    const std::vector<double> &q, const std::vector<double> &offset)>;
using RobotMovementCallback = std::function<void(bool accepted, int error)>;

/**
 * This interface provides functionality for requesting input or actions from
 * end users
 */
class ARCS_ABI_EXPORT UserInteraction
{
public:
    UserInteraction(UserInteraction &f);
    UserInteraction(UserInteraction &&f);
    virtual ~UserInteraction();

    /**
     * <p>
     * Request the end user to use the robot to define a robot position.
     * Override the
     * {@link RobotPositionCallback2#onOk(PositionParameters)} method to execute
     * code once the end user is done. Optionally override the {@link
     * RobotPositionCallback2#onCancel()} method to execute code if the end user
     * cancels the operation.
     * </p>
     *
     * <p>
     * This method is asynchronous, i.e. the method will return immediately.
     * Only when the end user is done, either
     * {@link RobotPositionCallback2#onOk(PositionParameters)} or {@link
     * RobotPositionCallback2#onCancel()} will be called.
     * </p>
     *
     * NOTE: This functionality should not be used in a toolbar contribution
     * (see {@link ToolbarService}).
     *
     * @param callback the instance callbacks will be made to.
     */
    void getUserDefinedRobotPosition(RobotPositionCallback callback);

    /**
     * 指定 tcpOfffset
     *
     * @param tcpOffset
     * @param callback
     */
    void getUserDefinedRobotPosition(const std::vector<double> &tcpOffset,
                                     RobotPositionCallback callback);

    /**
     * 指定 tcpOfffset
     *
     * @param tcpOffset
     * @param callback
     */
    void getUserDefinedRobotPosition(const std::string &tcpOffset,
                                     RobotPositionCallback callback);

    /**
     * 指定 tcpOfffset
     *
     * @param tcpOffset
     * @param callback
     */
    void getUserDefinedRobotPosition(const TCPPtr tcpOffset,
                                     RobotPositionCallback callback);

    /**
     * 获取机器人缩略图
     */
    QPixmap *getRobotThumb(const std::vector<double> &q);

    /**
     * <p>
     * This method provides a factory for creating keyboard inputs which are
     * used to configure a virtual keyboard/keypad and to request it to be
     * displayed for a Swing GUI component.
     * </p>
     *
     * NOTE: This functionality is only relevant for AuboCap nodes with a
     * Swing-based user interface (see {@link ProgramNodeService},
     * {@link InstallationNodeService} and {@link ToolbarService}).
     *
     * @return factory providing access to keyboard inputs.
     */
    KeyboardInputFactoryPtr getKeyboardInputFactory();

    int requestUserToMoveJoint(const std::vector<double> &q,
                               RobotMovementCallback callback);
    int requestUserToMoveJoint(const std::string &q,
                               RobotMovementCallback callback);

    int requestUserToMoveLine(const std::vector<double> &q,
                              RobotMovementCallback callback);

private:
    friend class DataSwitch;
    UserInteraction();
    void *d_{ nullptr };
};

} // namespace aubo_scope
} // namespace arcs

#endif // USERINTERACTION_H
